/*
  This code was written by Raymond Chi-Wing Wong.
  Please read the following.

    Raymond Chi-Wing Wong, Yufei Tao, Ada Wai-Chee Fu and Xiaokui Xiao. 
    "On Efficient Spatial Matching", 
    the 33rd International Conference on Very Large Data Bases (VLDB'07), 
    Vienna, Austria on 23-28 Sept, 2007 

*/
#include "closestPairTree.h"


void closestTree_mainIteration(node_type *rootO, node_type *rootP, twoData *aTwoData, rtree_info *rtreeInfo, assignment *aAssignment)
{
	int oid, pid;
	point *aPointO;
	point *aPointP;

	int matchWeight;

	triple *aTriple;

	node_type *aNodeO;
	node_type *aNodeP;

	int inputLevel;
	node_type *aLeafNode;

	XkPair aXkPair;

	aXkPair = XkClosestPair(rootO, rootP, rtreeInfo, rtreeInfo);

	aNodeO = aXkPair.m_pNodeO;
	aNodeP = aXkPair.m_pNodeP;
	oid = aNodeO->id;
	pid = aNodeP->id;

	aPointO = twoData_getOPoint(aTwoData, oid);
	aPointP = twoData_getPPoint(aTwoData, pid);

	if (aPointO->weight < aPointP->weight)
	{
		matchWeight = aPointO->weight;
	}
	else
	{
		matchWeight = aPointP->weight;
	}

	aPointO->weight -= matchWeight;
	aPointP->weight -= matchWeight;

	if (aPointO->weight == 0)
	{
		// remove aPointO from rootO
		inputLevel = 1;	//dummy
		aLeafNode = aNodeO;
		condenseTree(rootO, aLeafNode, inputLevel, rtreeInfo);
	}

	if (aPointP->weight == 0)
	{
		// remove aPointP from rootP
		inputLevel = 1;	//dummy
		aLeafNode = aNodeP;
		condenseTree(rootP, aLeafNode, inputLevel, rtreeInfo);
	}

	// assignment
	aTriple = triple_newValue(oid, pid, matchWeight);
	assignment_insert(aAssignment, &aTriple);

}


int closestTree_isEmpty(pointList *aPointList)
{
	int returnValue;

	int noOfPoint;
	point **pointList;
	point *aPoint;

	int i;

	noOfPoint = aPointList->noOfPoint;
	pointList = aPointList->list;

	returnValue = 1;
	for (i = 0; i < noOfPoint; i++)
	{
		aPoint = pointList[i];

		if (aPoint->weight != 0)
		{
			returnValue = 0;
			break;
		}
	}

	return returnValue;
}

assignment *closestTree_findAssignment(twoData *aTwoData, node_type *rootO, node_type *rootP, configure *aConfigure)
{
	assignment *returnValue;

	int noOfDim;

	int noOfPointO;
	int noOfPointP;

	pointList *O;
	pointList *P;

	int iterationNo;

	rtree_info *rtreeInfo;

	int isEmptyO, isEmptyP;

	twoData *cloneTwoData;

	rtreeInfo = aConfigure->rtreeInfo;
	noOfDim = rtreeInfo->dim;

	cloneTwoData = twoData_clone(aTwoData, noOfDim);

	O = cloneTwoData->O;
	P = cloneTwoData->P;

	noOfPointO = O->noOfPoint;
	noOfPointP = P->noOfPoint;

	isEmptyO = closestTree_isEmpty(O);
	isEmptyP = closestTree_isEmpty(P);

	returnValue = assignment_new();

	iterationNo = 0;
	while ((isEmptyP == 0) && (isEmptyO == 0))
	{
		// iteration
		closestTree_mainIteration(rootO, rootP, cloneTwoData, rtreeInfo, returnValue);

		if (iterationNo%100 == 0)
		{
			printf("Iteration No %d...\n", iterationNo);
		}
		

		// next iteration
		iterationNo++;
		isEmptyO = closestTree_isEmpty(O);
		isEmptyP = closestTree_isEmpty(P);
	}


	return returnValue;
}
