/*
  This code was written by Raymond Chi-Wing Wong.
  Please read the following.

    Raymond Chi-Wing Wong, Yufei Tao, Ada Wai-Chee Fu and Xiaokui Xiao. 
    "On Efficient Spatial Matching", 
    the 33rd International Conference on Very Large Data Bases (VLDB'07), 
    Vienna, Austria on 23-28 Sept, 2007 

*/

#ifndef STAT_H
#define STAT_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "assignment.h"
#include "stable.h"
#include "point.h"

struct statistics_s
{
	// chainLen
	int chainLen_min;
	int chainLen_max;
	double chainLen_totalLen;
	int chainLen_totalNoOfChain;

	// noOfNNQuery and noOfDeletion
	int totalNoOfNNQueryFromP;
	int totalNoOfNNQueryFromO;
	int totalNoOfNNQuery;

	double ratioPerOSize;
	double ratioPerPPlusOSize;

	int totalNoOfDeletionFromP;
	int totalNoOfDeletionFromO;
	int totalNoOfDeletion;

	// noOfCouple
	int totalNoOfCouple;

	int noOfCoupleForO_min;
	int noOfCoupleForO_max;
	int noOfCoupleForO_total;

	int noOfCoupleForP_min;
	int noOfCoupleForP_max;
	int noOfCoupleForP_total;

	// Matching cost and matching weight
	double totalMatchingCost;
	double totalMatchingWeight;

	// Rtree
	int noOfNodeRtreeO;
	int rtreeO_storage;
	int noOfNodeRtreeP;
	int rtreeP_storage;

	// prefList and rankList
	int prefList_storage;
	int rankList_storage; 

	// totalStorage
	int totalStorage;
};

typedef struct statistics_s statistics;

extern statistics *globalStat;

statistics *statistics_new();
void statistics_free(statistics **aStat);


void statistics_update_chainLen(statistics *aStat, int chainLen);


void statistics_rtreeNoOfNodeStorage(node_type *rootO, node_type *rootP, rtree_info *aInfo);
void statistics_endMain(configure *aConfigure, twoData *aTwoData, assignment *finalAssignment, assignmentSet *assignmentSetForO, assignmentSet *assignmentSetForP);

void statistics_print(FILE *filePtr);

#endif
